local RBS = LibStub("AceAddon-3.0"):NewAddon("RespecBindSaver", "AceConsole-3.0", "AceEvent-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local DualSpec = LibStub('LibDualSpec-1.0')
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

function table.copy(t)
  local t2 = {}
  for k,v in pairs(t) do
	t2[k] = v
  end
  return t2
end

function RBS:LoadBinds()
	if self.db.profile.gotBinds then
		self.changingBinds = true -- performance protection from saving binds on each trigger of UPDATE_BINDINGS during loading bindings
		local profilekeys = table.copy(self.db.profile.keys)
		
		-- build a table of currently used keys in game
		local gamekeys = {}
		for i=0,GetNumBindings() do
			local binds = {GetBinding(i)}
			table.remove(binds, 1) --we don't want to iterate over action name
			for p, key in pairs(binds) do
				table.insert(gamekeys, key)
			end
		end		
		
		-- replace all current keys with new actions or nil if not included in current profile
		for p, key in pairs(gamekeys) do
			SetBinding(key, profilekeys[key])
			profilekeys[key] = nil
		end

		-- import any leftover bindings
		for key, action in pairs(profilekeys) do
			if (profilekeys[key]) then
				SetBinding(key, action)
			end
		end		
		SaveBindings(GetCurrentBindingSet())
		self.changingBinds = false
	end
end

function RBS:SaveBinds()
	if not self.changingBinds then
		self.db.profile.keys = {}
		for i=0,GetNumBindings() do
			local binds = {GetBinding(i)}
			action = binds[1]
			table.remove(binds, 1) -- we don't want to iterate over action name
			for p, k in pairs(binds) do
				self.db.profile.keys[k] = action
			end
		end	
		self.db.profile.gotBinds = true
	end
end

function RBS:FirstRun() -- generate default profiles on first run
	local charname = UnitName("player", false)
	local realmname = GetRealmName()
	local specname, otherspecname
	local spec = GetActiveTalentGroup()
	if (spec == 1) then
		specname = "Primary" .. " - " .. charname .. " - " .. realmname
		otherspecname = "Secondary" .. " - " .. charname .. " - " .. realmname
	else
		specname = "Secondary" .. " - " .. charname .. " - " .. realmname
		otherspecname = "Primary" .. " - " .. charname .. " - " .. realmname
	end

	self.db:SetProfile(otherspecname)
	self:SaveBinds()
	self.db:SetProfile(specname)
	self:SaveBinds()
	
	self.db:SetDualSpecEnabled(true)
	self.db:SetDualSpecProfile(otherspecname)
		
	self.db:DeleteProfile("tmp", true)  -- delete temporary profile
		
	self.db.char.NotFirstRun = true
end

function RBS:OnInitialize()
--	if RBS_DB and (RBS_DB["MainSpec"] or RBS_DB["OffSpec"]) then RBS_DB = nil end -- clear old db if exists
	
	self.db = AceDB:New("RBS_DB", nil, "tmp")
	
	self.options = {
		type = "group",
		args = {
			gui = {
				name = "GUI",
				desc = "Opens the GUI window",
				type = "execute",
				func = 	function()
							InterfaceOptionsFrame_OpenToCategory(self.GUIhandler)
						end,
			}
		}
	}
	self.options.args.profile = AceDBOptions:GetOptionsTable(self.db)
	
	DualSpec:EnhanceDatabase(self.db, "RBS_DB")
	DualSpec:EnhanceOptions(self.options.args.profile, self.db)

	AceConfig:RegisterOptionsTable("RespecBindSaver", self.options, {"rbs"})
	AceConfig:RegisterOptionsTable("RBSgui", self.options.args.profile)	
	self.GUIhandler = AceConfigDialog:AddToBlizOptions("RBSgui", "RespecBindSaver")
end

function RBS:OnEnable()
	if not self.db.char.NotFirstRun then self:FirstRun() end

	self.db.RegisterCallback(self, "OnProfileChanged", "LoadBinds")
	self.db.RegisterCallback(self, "OnProfileCopied", "LoadBinds")
	self.db.RegisterCallback(self, "OnProfileReset", "SaveBinds")
	self.db.RegisterCallback(self, "OnNewProfile", "SaveBinds")
	
	self:RegisterEvent("UPDATE_BINDINGS", "SaveBinds")
end
